require 'thor'
require 'thor/group'

require 'asteroid/generator'

module Asteroid

  module Application

    class Instance < Thor
      desc "list", "create a asteroid application at NAME"
      def list()
        instances = Asteroid::Instance.all
        instances.each do |instance|
          puts [instance.id, instance.name, instance.server.provider.type].join("\t")
        end
      end

      desc "ssh [instance-id]", "run the astroid command"
      def ssh(i)
      end

      desc "destroy COMMAND", "destroy the instance"
      option :instance
      def destroy(instance_id, *rest)
        instance = Asteroid::Instance.find instance_id
        instance.destroy
      end

      desc "eval COMMAND", "run the astroid command"
      option :instance
      option :server, banner: '<server>'
      def eval(instance_id, *rest)
        instance = Asteroid::Instance.find instance_id
        instance.eval_command rest.join(' ')
      end
    end

    class Server < Thor
      desc "list", "list server configs"
      def list
        servers = Asteroid::Server.all
        servers.each do |server|
          puts [server.name, server.provider.type].join("\t")
        end
      end

      desc "create TYPE", "create an instance of a server"
      def create(name)
        server = Asteroid::Server.named name
        if server
          server.create_instance!
        else
          puts "Unknown server named #{name}"
        end
      end

    end

    class CLI < Thor    
      desc "instance SUBCOMMAND", "test in CLI"
      subcommand "instance", Instance

      desc "server SUBCOMMAND", "test in CLI"
      subcommand "server", Server
    end

  end


  class NewApplication < Thor
    
    desc "new NAME", "create a asteroid application at NAME"
    option :from
    def new(name)
      project_root = File.expand_path name

      generator = Asteroid::Generator.new project_root
      generator.generate!

      #puts "from: #{options[:from]}" if options[:from]
    end

  end
end