module Asteroid

  def self.environment=(env)
    @environment = env.to_sym
  end

  def self.environment
    @environment ||= ENV["ASTEROID_ENV"] || :development
  end

  def self.root=(root)
    @root = root
  end

  def self.root
    @root
  end

  class Config

    class << self

      def boot!
        if File.exists? self.secret_config_file
          @secrets = YAML::load_file self.secret_config_file || {}
        else
          @secrets = {}
        end

        # Load secret provider definition file
        if File.exists?(Config.secret_provider_file)
          eval File.read(Config.secret_provider_file)
        end

        # Set search paths
        Asteroid::FileReference.search_paths << Config.file_dir
        Asteroid::FileReference.search_paths << Config.secret_file_dir

        Asteroid::ScriptReference.search_paths << Config.script_dir
        Asteroid::ScriptReference.search_paths << Config.secret_script_dir

        Asteroid::KeyReference.search_paths << Config.secret_key_dir
      end

      def configure(root, options = {}, &block)
        options = default_config(root)
        options.merge! options
        options.each do |key, value|
          self.send "#{key}=", value
        end

        yield self if block_given?

        boot!
      end

      def secret
        @secrets
      end

      def template_engines
        @template_engines ||= {}
      end

      attr_accessor :namespace
      attr_accessor :asteroid_dir
      attr_accessor :script_dir
      attr_accessor :file_dir
      attr_accessor :server_dir
      attr_accessor :config_dir
      attr_accessor :secret_dir
      attr_accessor :secret_file
      attr_accessor :secret_config_dir
      attr_accessor :secret_config_file
      attr_accessor :secret_provider_file
      attr_accessor :secret_script_dir
      attr_accessor :secret_file_dir
      attr_accessor :secret_key_dir
      attr_accessor :secret_instance_dir
   
      def default_config(root)
        asteroid_dir = File.join(root, '/asteroid')
        config_dir = File.join(root, '/config')
        secret_dir = File.join(root, '/secrets')

        {
          asteroid_dir: asteroid_dir,
          script_dir: File.join(asteroid_dir, '/scripts'),
          file_dir: File.join(asteroid_dir, '/files'),
          server_dir: File.join(asteroid_dir, '/servers'),
          config_dir: config_dir,
          secret_dir: secret_dir,
          secret_config_dir: File.join(secret_dir, '/config'),
          secret_config_file: File.join(secret_dir, '/config', '/secrets.yml'),
          secret_provider_file: File.join(secret_dir, '/config', '/providers.rb'),
          secret_script_dir: File.join(secret_dir, '/scripts'),
          secret_file_dir: File.join(secret_dir, '/files'),
          secret_key_dir: File.join(secret_dir, '/keys'),
          secret_instance_dir: File.join(secret_dir, '/instances')
        }
      end

    end





    # self.domain = "example.com"
    # self.base_dir = File.expand_path(File.join(File.dirname(__FILE__), '../..'))
    # self.config_dir = File.join(base_dir, '/config')
    # self.initializer = File.join(config_dir, '/initializer.rb')
    # self.script_dir = File.join(config_dir, 'scripts')
    # self.file_dir = File.join(config_dir, 'files')
    # self.secrets_dir = File.join(base_dir, '/secrets')
    # self.ssh_key_dir = File.join(secrets_dir, '/ssh-keys')
    # self.password_dir = File.join(secrets_dir, '/passwords')
    # self.config_data_dir = File.join(secrets_dir, '/config')
    # self.secret_file_dir = File.join(secrets_dir, '/files')
    # self.secret_script_dir = File.join(secrets_dir, '/scripts')
    # self.instance_config_dir = File.join(secrets_dir, '/instance-config')
    # self.server_dir = File.join(config_dir, 'servers')
  end

end
