require 'tempfile'
require 'pathname'

module Asteroid
  class FileReference

    class << self
      def search_paths
        @search_paths ||= []
      end
    end

    private

      def path_is_absolute?(path)
        Pathname.new(path).absolute?
      end

    public

    attr_accessor :data

    def initialize(fname)
      if path_is_absolute?(fname)
        @filename = fname
      else 
        # Scan search paths looking for exact match
        self.class.search_paths.each do |search|
          break unless @filename.nil?
          proposed_filename = File.join(search, fname)
          if File.exists?(proposed_filename)
            @filename = proposed_filename
          end
        end

        # Scan search paths looking for partial match
        self.class.search_paths.each do |search|
          break unless @filename.nil?
          glob = File.join(search, fname + "*")
          matches = Dir[glob]
          if matches.size == 1
            @filename = matches.first
          elsif matches.size > 1
            raise "Ambiguous filename #{fname}"
          end
        end
      end

      if @filename.nil?
        raise "Could not find file #{fname}"
      end
    end

    def name
      File.basename @filename
    end

    def type
      name.split('.').last.to_sym
    end

    def exists?
      # should always be yes
      File.exists?(@filename)
    end

    def template?
      last = @filename.split('.').last
      if last
        Asteroid::Config.template_engines.keys.include?(last.to_sym)
      else
        false
      end
    end

    def filename
      @filename
    end

    def rendered_filename
      if template?
        @rendered_filename ||= begin
          file = Tempfile.new('config')
          file.write render
          file.rewind
          file.path
        end
      else
        filename
      end
    end

    def render(extra = nil)
      template_data = read
      rendered_template = Template.new(:erb).render(
        template_data, 
        data.merge(extra)
      )
    end

  end
end