require 'net/ssh'

require_relative './instance/vars'
require_relative './instance/scp'
require_relative './instance/ssh'
require_relative './instance/command'


module Asteroid

  class MissingAttributeError < StandardError
    def initialize(name)
      @attribute_name = name
    end

    def attribute
      @attribute_name
    end
  end

  class Instance

    attr_accessor :logger

    attr_reader :server, 
                :provider,
                :type,
                :id,
                :name,
                :ip_address,
                :namespace


    def initialize(options = {})
      options = options.dup

      o = options.dup

      @server = options.delete :server
      @type = options.delete :type
      @id = options.delete :id
      @name = options.delete :name
      @ip_address = options.delete :ip_address

      if @name =~ /:/
        @namespace, @name = @name.split(':', 2)
      end

      if @name && @type.nil?
        @type = name.split('-').first.to_sym
      end

      if @server.nil? && @type.nil?
        raise "Instance must have a server type :type attribute"
      elsif @server.nil?
        @server = Server.named @type
      end

      if @provider

      end

      @attributes = options

 
      server.provider.required_instance_attributes.each do |att|
        require_attribute att
      end

      # The type is added on the name
      @attributes[:type] ||= if @attributes[:name]
        @attributes[:name].match(/[a-z]+/)[0].to_sym
      end
    end

    def self.all
      instances = Provider.all.instances
      instances = instances.map do |info|
        new info
      end

      if Config.namespace
        instances = instances.delete_if do |i|
          i.namespace != Config.namespace
        end
      end

      instances
    end

    def self.first(type)
      all_with_type(type).first
    end

    def self.all_with_type(type)
      all.select{|i| i.type == type}
    end

    def password(username = nil)
      username ||= self["login.username"]
      password_filename = File.join(Asteroid::Config.password_dir, "/instance_#{@id}-user_#{username}")
      if File.exists? password_filename
        File.read(password_filename)
      else
        password = SecureRandom.hex(64)
        File.open(password_filename, 'w'){|f| f.write password}
        password
      end
    end

    def server_defaults
      server.instance_config
    end

    def server
      @server ||= Server.named self.type.to_s
    end

    def self.find(id)
      all.select{|i| i.id == id || i.id.to_s == id.to_s}.first
    end

    def env
      {
        "INSTANCE_PRIVATE_KEY" => self.server.ssh_key_filename,
        "INSTANCE_SSH_PORT" => self["ssh.port"].to_i,
        "INSTANCE_SSH_LOGIN" => self["login.username"],
        "INSTANCE_IP_ADDRESS" => self.ip_address,
        "INSTANCE_ID" => self.id,
      }
    end

    def destroy
      server.provider.destroy_instance self
    end

    private

      def require_attribute(name)
        if @attributes[name].nil?
          raise MissingAttributeError.new(name)
        end
      end

  end
end