require 'net/scp'

module Asteroid

  class MockSCP

    def initialize(fs)
      @fs = fs || {}
    end

    def upload!(from, to)
      @fs[to] = File.read(from)
    end

  end

  class Instance

    def scp_upload(from, to)
      scp.upload! from, to
    end

    def mock_file_system
      @mock_file_system ||= {}
    end

    def scp
      if server.provider.type == :mock
        return @mock_scp ||= MockSCP.new(mock_file_system)
      end

      if @scp && (@scp.session.transport.port.to_s != self["ssh.port"] || (@scp.session.transport.options[:user] != self["login.username"]))
        @scp = nil
      end
      @scp ||= Net::SCP.start(
        ip_address, 
        self["login.username"],
        port: self["ssh.port"].to_i,
        :keys => [server.ssh_key_filename]
      )
    end
  end  
end