require 'fileutils'

module Asteroid

  class Instance

    def [](k)
      val = config_get k
      val.nil? ? server_defaults[k] : val
    end

    def []=(k, v)
      config_set k, v
    end

    def config_get(k)
      data = load_config_data
      data && data[k]
    end

    def config_set(k, v)
      data = load_config_data
      data[k] = v
      save_config_data data
      v
    end

    def config_filename
      unless File.directory?(Asteroid::Config.secret_instance_dir)
        FileUtils.mkdir Asteroid::Config.secret_instance_dir
      end
      File.join(Asteroid::Config.secret_instance_dir, "/instance_#{@id}.yml")
    end

    def load_config_data
      begin
        YAML::load_file config_filename
      rescue
        {}
      end
    end

    def save_config_data(data)
      File.open(config_filename, 'w') {|f| f.write data.to_yaml }
    end
  end
  
end