
module Asteroid
  module Provider

    class DigitalOcean < Abstract

      def initialize(config = {})
        @config = config
        Digitalocean.client_id  = config[:client_id]
        Digitalocean.api_key    = config[:api_key]
      end

      def sizes
        Digitalocean::Size.all.sizes.map do |d|
          d.marshal_dump.merge(
            provider: self
          )
        end
      end

      def instances
        response = Digitalocean::Droplet.all
        response.droplets.map do |d|
          d.marshal_dump.merge(
            provider: self
          )
        end
      end

      def create_instance(server)

        instance_options = server.instance_options

        require_attribute instance_options, 
                          :size_id,
                          "needs size of droplet"

        require_attribute instance_options,
                          :image_id, 
                          "needs starting image for droplet"

        require_attribute instance_options, 
                          :region_id, 
                          "needs region for droplet"

        require_attribute instance_options, 
                          :ssh_key_ids, 
                          "needs SSH key registered for droplet"

        instance = Digitalocean::Droplet.create(
          name: server.generate_instance_name, 
          size_id: instance_options[:size_id], 
          image_id: instance_options[:image_id], 
          region_id: instance_options[:region_id], 
          ssh_key_ids: instance_options[:ssh_key_ids],
          private_networking: true
        )

        if instance.status == "OK"
          Instance.new instance.droplet.marshal_dump.merge(provider: self)
        else
          nil
        end
      end
   
      def destroy_instance(instance)
        Digitalocean::Droplet.destroy instance.id
      end

      def ssh_keys
        response = Digitalocean::SshKey.all
        response.ssh_keys.map do |d|
          d.marshal_dump.merge(
            provider: self
          )
        end
      end

      def create_ssh_key(config)
      end

      def reboot_instance(instance)
      end

      private

        def provider_name
          "DigitalOcean"
        end

        def instance_name
          "Droplet"
        end

    end
  end
end