require "fileutils"

module Asteroid
  module Provider
    class Mock < Abstract

      def self.ip_address
        (0..3).to_a.map{|a| rand(255)}.join(".")
      end

      def self.clear!
        FileUtils.rm filename if File.exists? filename
      end

      def self.filename
        File.join(File.dirname(__FILE__), '../../../world.yml')
      end

      def initialize(config = {})
        @config = config
        @config[:filename] ||= self.class.filename
        
        load_world
      end

      def instances
        @world[:instances] ||= []
      end
      
      def create_instance(server)
        instance = {
          name: server.generate_instance_name,
          id: SecureRandom.hex(5),
          ip_address: self.class.ip_address,
          type: server.type
        }

        self.instances << instance 
        save_world

        Instance.new instance.merge({
          provider: self,
          server: server
        })
      end

      def destroy_instance(instance)
        self.instances.delete_if do |i|
          i[:id] == i.id
        end
        save_world
      end

      private

        def load_world 
          begin
            @world = YAML::load_file @config[:filename]
          rescue
            @world = {}
          end
        end

        def save_world
          File.open(@config[:filename], 'w') do |f|
            f.write @world.to_yaml
          end
          load_world
        end

    end
  end
end